<?php


/**
 * @file
 * This is a restful service class for the Atlassian SSO solution crowd.
 *
 * @author
 * John P. Warren, Andrew Nagy
 */


/**
 * Provides a crowd REST service connection.
 */
class crowd_rest_client implements crowd_service {
  // Properties.
  protected $request_url;
  protected $request_headers;
  protected $request_method;
  protected $request_data;
  protected $request_service;


  /**
   * Constructor.
   *
   * Defines the base connection URL and other common properties.
   */
  function __construct() {
    // Create the full url for drupal_http_request().
    $server_address = variable_get('crowd_server', '');
    if (strpos($server_address, 'http://') !== FALSE) {
      $url_prefix = 'http://';
    }
    else {
      $url_prefix = 'https://';
    }
    if (!empty($url_prefix)) {
      $url_suffix = str_ireplace($url_prefix, '', $server_address); // ($server_data, $url);
    }
    else {
      watchdog('crowd', 'Crowd is not configured on this server!', array(), WATCHDOG_ERROR);
      drupal_set_message(t('Crowd is not configured on this server, please contact the site administrator for help.'));
    }
    $this->request_url = $url_prefix . variable_get('crowd_application', '') . ':' . variable_get('crowd_application_password', '') . '@' . $url_suffix . ':' . variable_get('crowd_port', '443') . variable_get('crowd_uri', '');
    $this->request_headers = array('Accept' => 'application/json', 'Content-Type' => 'application/xml');
    $this->request_method = 'POST';
  }


  /**
   * Authorize a user through the crowd server.
   *
   * @param string $username
   *   Crowd username
   * @param string $password
   *   User password
   * @return
   *   A keyed array with the Crowd token or FALSE
   */
  public function authorize($username, $password) {
    $service = '/session';
    $XML = new SimpleXMLElement("<?xml version='1.0' encoding='utf-8'?><authentication-context></authentication-context>");
    $XML->addChild('username', htmlspecialchars($username, ENT_NOQUOTES, 'UTF-8'));
    $XML->addChild('password', htmlspecialchars($password, ENT_NOQUOTES, 'UTF-8'));
    $vfs = $XML->addChild('validation-factors');
    $vf = $vfs->addChild('validation-factor');
    $vf->addChild('name', 'remote_address');
    $vf->addChild('value', ip_address());
    //@todo add explicit json header
    $this->request_method = 'POST';
    $final_url = $this->request_url . $service;
    $options = array(
      'headers' => $this->request_headers,
      'method' => $this->request_method,
      'data' => $XML->asXML(),
      'max_redirects' => 3,
      'timeout' => 120.0,
    );
    $result = drupal_http_request($final_url, $options);

    if ($result->code == '204' && $result->status_message == 'DELETE') {
      return TRUE;
    }
    if ($result->code == '200' || $result->code == '201') {
      $data = json_decode($result->data);
      $return_data = array('token' => $data->token);
      drupal_set_message(t('Successfully authenticated @name via crowd.', array('@name' => $data->user->name)));
      return $return_data;
    }
    else {
      watchdog('crowd', 'Restful service return http code %code.', array('%code' => $result->code), WATCHDOG_WARNING);
      drupal_set_message(t('Unable to log into crowd server, check your username/password and try again.'), 'error');
      return FALSE;
    }
  }


  /**
   * Log the user out from the crowd server.
   *
   * @param string $crowd_token
   *   The active crowd token for a user.
   * @return
   *   Boolen TRUE on success for FALSE on failure.
   */
  public function logout($crowd_token) {
    global $user;
    $service = '/session';
    $final_url = $this->request_url . $service . '/' . $crowd_token;
    $this->request_method = 'DELETE';
    $this->request_data = '';
    // Invalidate the token for this user for all application clients in Crowd.
    $options = array(
      'headers' => $this->request_headers,
      'method' => $this->request_method,
      'data' => $this->request_data,
      'max_redirects' => 3,
      'timeout' => 120.0,
    );
    $result = drupal_http_request($final_url, $options);
    if ($result->code == '204' && $result->status_message == 'No Content') {
      return TRUE;
    }
    else {
      watchdog('crowd', 'Logout of crowd failed, messages: @data', array('@data' => serialize($result)), WATCHDOG_ERROR);
      return FALSE;
    }
  }


  /**
   * Get user data based on an active session token.
   *
   * @param string $user_token
   *   The active crowd token for a user.
   * @return
   *   An array containing the user data points or FALSE on failure.
   */
  public function user_get($user_token) {
    $service = '/session';
    $this->request_method = 'GET';
    $this->request_data = url($user_token, array('query' => array('expand' => 'user')));
    $final_url = $this->request_url . $service . $this->request_data;
    $options = array(
      'headers' => $this->request_headers,
      'method' => $this->request_method,
      'max_redirects' => 3,
      'timeout' => 120.0,
    );
    $result = drupal_http_request($final_url, $options);
    $this->request_method = 'POST';
    if ($result->code == '200' || $result->code =='201') {
      $crowd_user = json_decode($result->data);
      $crowd_data = array();
      $crowd_data['username'] = $crowd_user->user->name;
      $crowd_data["email"] = $crowd_user->user->email;
      $crowd_data["active"] = $crowd_user->user->active;
      return $crowd_data;
    }
    else {
      watchdog('crowd', 'Crowd authentication failed to get the user object for user %username. Error: %data', array('%username' => $username, '%data' => serialize($result)), WATCHDOG_ERROR);
      drupal_set_message(t('There has been a problem with your user status on the Crowd server, please see your Atlassian Crowd server administrator for help.'), 'error');
      return FALSE;
    }
  }


  /**
   * Check to see if the user is already logged into Crowd.
   *
   * @param string $user_token
   *   The active crowd token for a user.
   * @return
   *   Boolean TRUE if the user is logged-in to crowd or FALSE if not.
   */
  public function crowd_is_logged_in($user_token) {
    $service = '/session';
    $this->request_method = 'GET';
    $this->request_headers = array('Accept' => 'application/json', 'Content-Type' => 'application/xml');
    $this->request_data = url($user_token);
    $final_url = $this->request_url . $service . $this->request_data;
    $options = array(
      'headers' => $this->request_headers,
      'method' => $this->request_method,
      'max_redirects' => 3,
      'timeout' => 120.0,
    );
    $result = drupal_http_request($final_url, $options);
    $this->request_method = 'POST';
    $data = json_decode($result->data);
    //var_dump($data);
    //die;
    if (isset($data->user->active) ) { // Should always be true or false
      return TRUE;
    }
    else {
      return FALSE;
    }
  }


  /**
   * Get all active groups from the crowd server.
   *
   * @return
   *   A serial array of the groupnames or FALSE on failure.
   */
  public function list_groups() {
    $service = '/search';
    $function = url('', array('query' => array('entity-type' => 'group')));
    $XML = new SimpleXMLElement("<?xml version='1.0' encoding='utf-8'?><property-search-restriction></property-search-restriction>");
    $prop = $XML->addChild('property');
    $prop->addChild('name', 'active');
    $prop->addChild('type', 'BOOLEAN');
    $XML->addChild('match-mode', 'EXACTLY_MATCHES');
    $XML->addChild('value', 'true');
    if ($this->request_method != 'POST') {
      $this->request_method = 'POST';
    }
    $final_url = $this->request_url . $service . str_ireplace('/', '', $function);
    $options = array(
      'headers' => $this->request_headers,
      'method' => $this->request_method,
      'data' => $XML->asXML(),
      'max_redirects' => 3,
      'timeout' => 120.0,
    );
    $result = drupal_http_request($final_url, $options);
    if ($result->code == '200' || $result->code =='201') {
      $data = json_decode($result->data);
      $results = array();
      foreach ($data->groups as $key => $value) {
        $results[] = $value->name;
      }
      return $results;
    }
    else {
      watchdog('crowd', 'Crowd group listing failed. Error: %data', array('%data' => serialize($result)), WATCHDOG_ERROR);
      return FALSE;
    }
  }


  /**
   * Get the crowd groups for a specific user.
   *
   * @param string $name
   *   The crowd username.
   * @return
   *   An array containing all of the given user's groups or FALSE on failure.
   */
  public function list_user_groups($name) {
    $service = '/user/group/nested';
    $url_query = str_ireplace('/', '', url('', array('query' => array('username' => $name))));
    $this->request_data .= '';
    if ($this->request_method != 'GET') {
      $this->request_method = 'GET';
    }
    $final_url = $this->request_url . $service . $url_query;
    $options = array(
      'headers' => $this->request_headers,
      'method' => $this->request_method,
      'data' => $this->request_data,
      'max_redirects' => 3,
      'timeout' => 120.0,
    );
    $result = drupal_http_request($final_url, $options);
    $this->request_method = 'POST';
    if ($result->code == '200' || $result->code =='201') {
      $user_crowd_data = json_decode($result->data);
      $user_crowd_groups = array();
      foreach ($user_crowd_data->groups as $key => $value) {
        $user_crowd_groups[] = $value->name;
      }
      return $user_crowd_groups;
    }
    else {
      watchdog('crowd', 'Crowd user group listing failed. Error: %data', array('%data' => serialize($result)), WATCHDOG_ERROR);
      drupal_set_message(t('There has been a problem with your user status on the Crowd server, please see your Atlassian Crowd server administrator for help.'), 'error');
      return FALSE;
    }
  }
}
